'use strict';

var core = require('@formkit/core');

// packages/zod/src/zod.ts
function createMessageName(node) {
  if (typeof node.props.validationLabel === "function") {
    return node.props.validationLabel(node);
  }
  return node.props.validationLabel || node.props.label || node.props.name || String(node.name);
}
function createZodPlugin(zodSchema, submitCallback) {
  const zodValidationSet = /* @__PURE__ */ new Set();
  const zodValidationListeners = /* @__PURE__ */ new Map();
  const zodPlugin = (node) => {
    if (node.props.type !== "form")
      return false;
    node.ledger.count("existingValidation", (message) => {
      if (message.type === "validation" || message.type === "error") {
        return !message.key.endsWith(":zod");
      }
      return false;
    });
    node.on("created", () => {
      node.extend("setZodErrors", {
        get: (node2) => async (zodError) => {
          if (!zodError)
            return;
          await node2.settled;
          const [formErrors, fieldErrors] = zodErrorToFormKitErrors(
            zodError,
            node2
          );
          node2.setErrors(fieldErrors, formErrors);
        },
        set: false
      });
    });
    node.on("destroyed", () => {
      zodValidationSet.clear();
      zodValidationListeners.clear();
    });
    let commitTimout = 0;
    let start = Date.now();
    node.on("commit", ({ payload }) => {
      clearTimeout(commitTimout);
      const now = Date.now();
      if (now - start > 600) {
        start = now;
        performZodValidation(payload, node);
      }
      commitTimout = setTimeout(() => {
        performZodValidation(payload, node);
      }, 150);
    });
    node.on("message-added", (message) => {
      if (message.payload.type === "state" && message.payload.key === "submitted") {
        performZodValidation(node.value, node);
      }
    });
    return false;
  };
  function performZodValidation(payload, node) {
    const zodResults = zodSchema.safeParse(payload);
    if (!zodResults.success) {
      setFormValidations(zodResults.error, node);
    } else {
      zodValidationSet.forEach((node2) => {
        node2.store.remove(`${node2.address.slice(1).join(".")}:zod`);
        const receipt = zodValidationListeners.get(node2);
        node2.off(receipt);
        zodValidationListeners.delete(node2);
      });
      zodValidationSet.clear();
    }
  }
  async function submitHandler(payload, node) {
    const zodResults = await zodSchema.safeParseAsync(payload);
    if (!zodResults.success) {
      setFormValidations(zodResults.error, node);
    } else {
      await submitCallback(zodResults.data, node);
    }
  }
  function setFormValidations(zodErrors, node) {
    if (!node)
      return;
    const [formErrors, InputErrors] = zodErrorToFormKitErrors(zodErrors, node);
    const oldZodValidationSet = new Set(zodValidationSet);
    Object.entries(InputErrors).map((issue) => {
      const [path, message] = issue;
      const targetNode = node.at(path);
      if (targetNode) {
        oldZodValidationSet.delete(targetNode);
        const existingValidationCount = targetNode.ledger.value("existingValidation");
        if (existingValidationCount === 0) {
          if (!targetNode.store[`${path}:zod`]) {
            const validationListener = targetNode.on(
              "unsettled:existingValidation",
              () => {
                targetNode.store.remove(`${path}:zod`);
                zodValidationSet.delete(targetNode);
              }
            );
            zodValidationListeners.set(targetNode, validationListener);
          }
          targetNode.store.set(
            /* @__PURE__ */ core.createMessage({
              blocking: true,
              type: "validation",
              key: `${path}:zod`,
              value: message,
              meta: {
                i18nArgs: [
                  {
                    node: targetNode,
                    name: createMessageName(targetNode),
                    args: [issue]
                  }
                ],
                messageKey: "zod"
              }
            })
          );
          zodValidationSet.add(targetNode);
        }
      }
    });
    if (node?.context?.state.submitted) {
      node.setErrors([], formErrors);
    }
    oldZodValidationSet.forEach((node2) => {
      node2.store.remove(`${node2.address.slice(1).join(".")}:zod`);
      zodValidationSet.delete(node2);
    });
  }
  function zodErrorToFormKitErrors(zodError, node) {
    const fieldErrors = {};
    const formErrors = [];
    zodError.issues.forEach((error) => {
      const path = error.path.join(".");
      const targetNode = node.at(path);
      if (targetNode) {
        if (!fieldErrors[path]) {
          fieldErrors[path] = error.message;
        }
      } else {
        formErrors.push(`${path}: ${error.message}`);
      }
    });
    return [formErrors, fieldErrors];
  }
  return [zodPlugin, submitHandler];
}

exports.createZodPlugin = createZodPlugin;
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.cjs.map